@@ -1,6 +1,6 @@
 /* parser.c -- convert the command line args into an expression tree.
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 2000, 2001, 2003, 
-                 2004, 2005, 2006 Free Software Foundation, Inc.
+                 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,12 +18,14 @@
    USA.
 */
 
+#include <config.h>
 
 #include "defs.h"
 #include <ctype.h>
 #include <math.h>
 #include <assert.h>
 #include <pwd.h>
+#include <errno.h>
 #include <grp.h>
 #include <fnmatch.h>
 #include "modechange.h"
@@ -911,10 +913,65 @@ static boolean
 parse_gid (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
   struct predicate *p = insert_num (argv, arg_ptr, entry);
-  p->est_success_rate = (p->args.numinfo.l_val < 100) ? 0.99 : 0.2;
-  return p;
+  if (p)
+    {
+      p->est_success_rate = (p->args.numinfo.l_val < 100) ? 0.99 : 0.2;
+      return true;
+    }
+  else
+    {
+      return false;
+    }
 }
 
+
+static int
+safe_atoi (const char *s)
+{
+  long lval;
+  char *end;
+  
+  errno = 0;
+  lval = strtol(s, &end, 10);
+  if ( (LONG_MAX == lval) || (LONG_MIN == lval) )
+    {
+      /* max/min possible value, or an error. */
+      if (errno == ERANGE)
+	{
+	  /* too big, or too small. */
+	  error(1, errno, "%s", s);
+	}
+      else
+	{
+	  /* not a valid number */
+	  error(1, errno, "%s", s);
+	}
+      /* Otherwise, we do a range chack against INT_MAX and INT_MIN
+       * below.
+       */
+    }
+  
+  if (lval > INT_MAX || lval < INT_MIN)
+    {
+      /* The number was in range for long, but not int. */
+      errno = ERANGE;
+      error(1, errno, "%s", s);
+    }
+  else if (*end)
+    {
+      error(1, errno, "Unexpected suffix %s on %s",
+	    quotearg_n_style(0, options.err_quoting_style, end),
+	    quotearg_n_style(1, options.err_quoting_style, s));
+    }
+  else if (end == s)
+    {
+      error(1, errno, "Expected an integer: %s", 
+	    quotearg_n_style(0, options.err_quoting_style, s));
+    }
+  return (int)lval;
+}
+
+
 static boolean
 parse_group (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
@@ -937,15 +994,14 @@ parse_group (const struct parser_table* entry, char **argv, int *arg_ptr)
 	    {
 	      if (groupname[gid_len] == 0)
 		{
-		  gid = atoi (groupname);
-#warning "no error checking on the previous line"
+		  gid = safe_atoi (groupname);
 		}
 	      else
 		{
 		  /* XXX: no test in test suite for this */
 		  error(1, 0, _("%s is not the name of an existing group and"
-				" it does not look like a numeic group ID because "
-				"it has the unexpected suffix %s"),
+				" it does not look like a numeric group ID "
+				"because it has the unexpected suffix %s"),
 			quotearg_n_style(0, options.err_quoting_style, groupname),
 			quotearg_n_style(1, options.err_quoting_style, groupname+gid_len));
 		  return false;
@@ -1111,9 +1167,18 @@ static boolean
 parse_inum (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
   struct predicate *p =  insert_num (argv, arg_ptr, entry);
-  /* inode number is exact match only, so very low proportions of files match */
-  p->est_success_rate = 1e-6;
-  return p;
+  if (p)
+    {
+      /* inode number is exact match only, so very low proportions of
+       * files match
+       */
+      p->est_success_rate = 1e-6;
+      return true;
+    }
+  else
+    {
+      return false;
+    }
 }
 
 /* -ipath is deprecated (at RMS's request) in favour of 
@@ -1157,13 +1222,20 @@ static boolean
 parse_links (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
   struct predicate *p = insert_num (argv, arg_ptr, entry);
-  if (p->args.numinfo.l_val == 1)
-    p->est_success_rate = 0.99;
-  else if (p->args.numinfo.l_val == 2)
-    p->est_success_rate = 0.01;
+  if (p)
+    {
+      if (p->args.numinfo.l_val == 1)
+	p->est_success_rate = 0.99;
+      else if (p->args.numinfo.l_val == 2)
+	p->est_success_rate = 0.01;
+      else
+	p->est_success_rate = 1e-3;
+      return true;
+    }
   else
-    p->est_success_rate = 1e-3;
-  return p;
+    {
+      return false;
+    }
 }
 
 static boolean
@@ -1201,8 +1273,7 @@ insert_depthspec(const struct parser_table* entry, char **argv, int *arg_ptr,
       depth_len = strspn (depthstr, "0123456789");
       if ((depth_len > 0) && (depthstr[depth_len] == 0))
 	{
-	  (*limitptr) = atoi (depthstr);
-#warning "consider replacing all calls to atio with a utility function"
+	  (*limitptr) = safe_atoi (depthstr);
 	  if (*limitptr >= 0)
 	    {
 	      return parse_noop(entry, argv, arg_ptr);
@@ -2208,8 +2279,15 @@ static boolean
 parse_uid (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
   struct predicate *p = insert_num (argv, arg_ptr, entry);
-  p->est_success_rate = (p->args.numinfo.l_val < 100) ? 0.99 : 0.2;
-  return p;
+  if (p)
+    {
+      p->est_success_rate = (p->args.numinfo.l_val < 100) ? 0.99 : 0.2;
+      return true;
+    }
+  else
+    {
+      return false;
+    }
 }
 
 static boolean
@@ -2261,10 +2339,9 @@ parse_user (const struct parser_table* entry, char **argv, int *arg_ptr)
 	{
 	  int uid_len = strspn (username, "0123456789");
 	  if (uid_len && (username[uid_len]==0))
-	    uid = atoi (username);
+	    uid = safe_atoi (username);
 	  else
 	    return false;
-#warning "no success checking has occured in atoi on previous line"
 	}
       our_pred = insert_primary (entry);
       our_pred->args.uid = uid;
@@ -2778,10 +2855,10 @@ static void
 check_path_safety(const char *action, char **argv)
 {
   const char *path = getenv("PATH");
+  char *s;
 
   (void)argv;
   
-  char *s;
   s = next_element(path, 1);
   while ((s = next_element ((char *) NULL, 1)) != NULL)
     {
@@ -2793,7 +2870,10 @@ check_path_safety(const char *action, char **argv)
       else if ('/' != s[0])
 	{
 	  /* Relative paths are also dangerous in $PATH. */
-	  error(1, 0, _("The ralative path %s is included in the PATH environment variable, which is insecure in combination with the %s action of find.  Please remove that entry from $PATH"),
+	  error(1, 0, _("The relative path %s is included in the PATH "
+			"environment variable, which is insecure in "
+			"combination with the %s action of find.  "
+			"Please remove that entry from $PATH"),
 		safely_quote_err_filename(0, s),
 		action);
 	}
@@ -3096,19 +3176,22 @@ parse_time (const struct parser_table* entry, char *argv[], int *arg_ptr)
   struct predicate *our_pred;
   struct time_val tval;
   enum comparison_type comp;
-  const char *timearg;
+  const char *timearg, *orig_timearg;
   const char *errmsg = "arithmetic overflow while converting %s days to a number of seconds";
   time_t origin;
 
   if (!collect_arg(argv, arg_ptr, &timearg))
     return false;
+  orig_timearg = timearg;
 
   /* Decide the origin by previewing the comparison type. */
   origin = options.cur_day_start;
 
   if (get_comp_type(&timearg, &comp))
     {
-      /* Remember, we invert the sense of the comparison, so this tests against COMP_LT instead of COMP_GT... */
+      /* Remember, we invert the sense of the comparison, so this tests 
+       * against COMP_LT instead of COMP_GT... 
+       */
       if (COMP_LT == tval.kind)      
 	{
 	  uintmax_t expected = origin + (DAYSECS-1);
@@ -3119,11 +3202,14 @@ parse_time (const struct parser_table* entry, char *argv[], int *arg_ptr)
 		    _("arithmetic overflow when trying to calculate the end of today"));
 	    }
 	}
-      /* We discard the value of comp here, as get_relative_timestamp
-       * will set tval.kind. 
-       */
     }
-  
+  /* We discard the value of comp here, as get_relative_timestamp
+   * will set tval.kind.  For that to work, we have to restore
+   * timearg so that it points to the +/- prefix, if any.  get_comp_type()
+   * will have advanced timearg, so we restore it.
+   */
+  timearg = orig_timearg;
+
   if (!get_relative_timestamp(timearg, &tval, origin, DAYSECS, errmsg))
     return false;
 
@@ -3145,10 +3231,9 @@ parse_time (const struct parser_table* entry, char *argv[], int *arg_ptr)
       fprintf (stderr, "%ju %s", (uintmax_t) our_pred->args.reftime.ts.tv_sec, ctime (&t));
       if (tval.kind == COMP_EQ)
 	{
-	  t = our_pred->args.reftime.ts.tv_sec += DAYSECS;
+	  t = our_pred->args.reftime.ts.tv_sec + DAYSECS;
 	  fprintf (stderr, "                 <  %ju %s",
-		   (uintmax_t) our_pred->args.reftime.ts.tv_sec, ctime (&t));
-	  our_pred->args.reftime.ts.tv_sec -= DAYSECS;
+		   (uintmax_t) t, ctime (&t));
 	}
     }
   
