@@ -17,6 +17,7 @@
    USA.
 */
 
+#include <config.h>
 #include "defs.h"
 
 #ifdef HAVE_FCNTL_H
@@ -31,6 +32,7 @@
 #include <ctype.h>
 #include <string.h>
 #include <limits.h>
+#include <errno.h>
 #include <assert.h>
 
 #include "xalloc.h"
@@ -257,6 +259,11 @@ check_nofollow(void)
   struct utsname uts;
   float  release;
 
+  if (0 == O_NOFOLLOW)
+    {
+      return false;
+    }
+  
   if (0 == uname(&uts))
     {
       /* POSIX requires that atof() ignore "unrecognised suffixes". */
