@@ -1,5 +1,5 @@
 /* listfile.c -- display a long listing of a file
-   Copyright (C) 1991, 1993, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1993, 2000, 2004, 2005, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,62 +21,30 @@
 # include <config.h>
 #endif
 
-#ifndef __GNUC__
-# if HAVE_ALLOCA_H
-#  include <alloca.h>
-# else
-#  ifdef _AIX
- #  pragma alloca
-#  else
-#   ifdef _WIN32
-#    include <malloc.h>
-#    include <io.h>
-#   else
-#    ifndef alloca
-char *alloca ();
-#    endif
-#   endif
-#  endif
-# endif
-#endif
+#include <alloca.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <pwd.h>
 #include <grp.h>
 #include <time.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <string.h>
+#include <unistd.h> /* for readlink() */
 #include <openat.h>
+
 #include "human.h"
 #include "xalloc.h"
 #include "pathmax.h"
 #include "error.h"
 #include "filemode.h"
+#include "dircallback.h"
 
 #include "listfile.h"
 
-#if HAVE_STRING_H || STDC_HEADERS
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-
-
-/* The presence of unistd.h is assumed by gnulib these days, so we 
- * might as well assume it too. 
- */
-#include <unistd.h> /* for readlink() */
-
-
-#if STDC_HEADERS
-# include <stdlib.h>
-#else
-char *getenv ();
-extern int errno;
-#endif
-
 /* Since major is a function on SVR4, we can't use `ifndef major'.  */
 #ifdef MAJOR_IN_MKDEV
 #include <sys/mkdev.h>
@@ -189,11 +157,6 @@ extern int errno;
 # define ST_NBLOCKSIZE 512
 #endif
 
-#ifndef _POSIX_VERSION
-struct passwd *getpwuid ();
-struct group *getgrgid ();
-#endif
-
 #ifdef major			/* Might be defined in sys/types.h.  */
 #define HAVE_MAJOR
 #endif
@@ -204,7 +167,7 @@ struct group *getgrgid ();
 #undef HAVE_MAJOR
 
 
-static void print_name (register char *p, FILE *stream, int literal_control_chars);
+static void print_name (register const char *p, FILE *stream, int literal_control_chars);
 
 extern char * getgroup (gid_t gid);
 extern char * getuser (uid_t uid);
@@ -228,7 +191,7 @@ file_blocksize(const struct stat *p)
    STREAM is the stdio stream to print on.  */
 
 void
-list_file (char *name,
+list_file (const char *name,
 	   int dirfd, 
 	   char *relname,
 	   const struct stat *statp,
@@ -359,14 +322,14 @@ list_file (char *name,
 
 
 static void
-print_name_without_quoting (char *p, FILE *stream)
+print_name_without_quoting (const char *p, FILE *stream)
 {
   fprintf(stream, "%s", p);
 }
 
 
 static void
-print_name_with_quoting (register char *p, FILE *stream)
+print_name_with_quoting (register const char *p, FILE *stream)
 {
   register unsigned char c;
 
@@ -415,7 +378,7 @@ print_name_with_quoting (register char *p, FILE *stream)
     }
 }
 
-static void print_name (register char *p, FILE *stream, int literal_control_chars)
+static void print_name (register const char *p, FILE *stream, int literal_control_chars)
 {
   if (literal_control_chars)
     print_name_without_quoting(p, stream);
