@@ -1,19 +1,23 @@
-/* pred.c -- execute the expression tree.
-   Copyright (C) 1990, 1991, 1992, 1993, 1994, 2000, 
-                 2003, 2004 Free Software Foundation, Inc.
+/* qmark.c -- quote 'dangerous' filenames
 
-   This program is free software: you can redistribute it and/or modify
+   Copyright (C) 2005 Free Software Foundation, Inc.  
+   Derived from courutils' ls.c:
+   Copyright (C) 85, 88, 90, 91, 1995-2005 Free Software Foundation, Inc.  
+
+   This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-   
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+   USA.
 */
 
 #ifdef HAVE_CONFIG_H
